@@ -2,26 +2,26 @@
 # -*- coding: utf-8 -*-
 
 import importlib
+import time
 import types
 import ssl
 import sys
 import urllib3
-import http
-import urllib3.util as urllib_util
-from urllib3.exceptions import HTTPError
+from urllib3.util import SSLContext
+
+from urllib3.exceptions import HTTPError, ProtocolError, MaxRetryError, ReadTimeoutError
 import xml.etree.ElementTree as etree
+from collections import OrderedDict
+from xml.sax.saxutils import escape
+
 try:
     # Python 3
+    import http
     import http.client as httplib
 except ImportError:
     # Python 2
     import httplib
 
-from collections import OrderedDict
-from xml.sax.saxutils import escape
-import time
-from urllib3.exceptions import ProtocolError, MaxRetryError, ReadTimeoutError
-
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 def get_http_status_message(status_code):
@@ -48,6 +48,9 @@ else:
     raise
 
 
+class UnauthorizedError(Exception):
+    pass
+
 class XMLWrapper:
     @staticmethod
     def parseXML(xml_str):
@@ -359,7 +362,7 @@ class CloudShellAPIError(Exception):
 
 
 class CommonAPISession:
-    def __init__(self, host, username, password, domain, connection_pool_size=25):
+    def __init__(self, host, username, password, domain, connection_pool_size=25, **kwargs):
         self.host = host
         self.username = username
         self.password = password
@@ -369,11 +372,11 @@ class CommonAPISession:
             ssl_protocol = ssl.PROTOCOL_SSLv23
         else:
             ssl_protocol = ssl.PROTOCOL_TLS
-        ctx = urllib_util.SSLContext(ssl_protocol)
+        ctx = SSLContext(ssl_protocol)
         ctx.check_hostname = False
         ctx.verify_mode = ssl.CERT_NONE
 
-        self._http = urllib3.PoolManager(num_pools=connection_pool_size,  ssl_context=ctx)
+        self._http = urllib3.PoolManager(num_pools=connection_pool_size,  ssl_context=ctx, **kwargs)
 
     def _parseXML(self, xml_str):
         return etree.fromstring(xml_str)
@@ -416,7 +419,7 @@ class CommonAPISession:
         operation_url = str(self.url + operation)
         response = retry(self._http.request, ProtocolError, attempts, 1,
                          method="POST", url=operation_url, body=message.encode('utf-8'), headers=request_headers)
-        
+
         self._check_http_response(response)
         
         return response
@@ -427,8 +430,16 @@ class CommonAPISession:
                 status_description = get_http_status_message(response.status)
             except ValueError:
                 status_description = 'Unknown'
-            raise Exception("({}) [{}] reason:{}".format(status_description, response.status, response.reason))
-        
+
+            if response.status == 401:
+                raise UnauthorizedError(
+                    "({}) [{}] reason:{}".format(status_description, response.status,
+                                                 response.reason))
+
+            raise Exception(
+                "({}) [{}] reason:{}".format(status_description, response.status,
+                                             response.reason))
+
         if not response.data:
             raise Exception(response.reason)
 
